import 'dart:async';

import 'package:dart_vlc/dart_vlc.dart';
import 'package:jellyflut/globals.dart';
import 'package:jellyflut/models/jellyfin/item.dart';
import 'package:jellyflut/providers/streaming/streamingProvider.dart';
import 'package:jellyflut/screens/stream/CommonStream/CommonStream.dart';
import 'package:jellyflut/services/streaming/streamingService.dart';
import 'package:rxdart/rxdart.dart';

class CommonStreamVLCComputer {
  static final streamingProvider = StreamingProvider();
  static final List<Timer> timers = [];
  final Player player;

  const CommonStreamVLCComputer({required this.player});

  static Future<Player> setupData({required Item item}) async {
    final streamURL = await item.getItemURL(directPlay: true);
    final _player = Player(id: videoPlayerId, commandlineArguments: [
      '--start-time=${Duration(microseconds: item.getPlaybackPosition()).inSeconds}',
    ]);
    final media = Media.network(streamURL);
    _player.open(media);

    // create timer to save progress
    final timer = _startProgressTimer(item, _player);
    streamingProvider.timer?.cancel();
    streamingProvider.setTimer(timer);

    // create common stream controller
    final commonStream = CommonStream.parseVlcComputerController(
        player: _player, listener: () => {});

    streamingProvider.setCommonStream(commonStream);
    return Future.value(_player);
  }

  static Future<Player> setupDataFromUrl({required String url}) async {
    final player = Player(
      id: videoPlayerId,
    );
    final media = Media.network(url);
    player.open(media);

    // create common stream controller
    final commonStream = CommonStream.parseVlcComputerController(
        player: player, listener: () => {});

    StreamingProvider().setCommonStream(commonStream);
    return Future.value(player);
  }

  void addListener(void Function() listener) {
    final timer =
        Timer.periodic(Duration(milliseconds: 100), (i) => listener());
    timers.add(timer);
  }

  void removeListener() {
    timers.forEach((t) => t.cancel());
  }

  static Timer _startProgressTimer(Item item, Player _player) {
    return Timer.periodic(
        Duration(seconds: 15),
        (Timer t) => StreamingService.streamingProgress(item,
            canSeek: _player.playback.isSeekable,
            isMuted: _player.general.volume > 0 ? true : false,
            isPaused: !_player.playback.isPlaying,
            positionTicks: _player.position.position?.inMicroseconds ?? 0,
            volumeLevel: _player.general.volume.round(),
            subtitlesIndex: 0));
  }

  BehaviorSubject<Duration> positionStream() {
    final streamController = BehaviorSubject<Duration>();
    player.positionStream.listen((PositionState positionState) =>
        streamController.add(positionState.position ?? Duration(seconds: 0)));
    return streamController;
  }

  BehaviorSubject<Duration> durationStream() {
    final streamController = BehaviorSubject<Duration>();
    player.positionStream.listen((PositionState positionState) =>
        streamController.add(positionState.duration ?? Duration(seconds: 0)));
    return streamController;
  }

  BehaviorSubject<bool> playingStateStream() {
    final streamController = BehaviorSubject<bool>();
    player.playbackStream
        .listen((PlaybackState event) => streamController.add(event.isPlaying));
    return streamController;
  }

  void stopPlayer() {
    player.stop();
    player.dispose();
  }
}
